@@ -57,9 +57,9 @@ namespace Beagle.Filters {
 		private HtmlDocument doc;
 
 		// delegate types
-		public delegate int AppendTextCallback (string s);
+		public delegate bool AppendTextCallback (string s);
 		public delegate void AddPropertyCallback (Beagle.Property p);
-		public delegate void AppendSpaceCallback ();
+		public delegate bool AppendSpaceCallback ();
 		public delegate void HotCallback ();
 
 		// delegates
@@ -71,12 +71,15 @@ namespace Beagle.Filters {
 		private new HotCallback HotUp;
 		private new HotCallback HotDown;
 
+		// 1: Add meta keyword fields as meta:key
+		private int version = 1;
+
 		public FilterHtml (bool register_filter)
 		{
 			if (register_filter) {
-				// 1: Add meta keyword fields as meta:key
-				SetVersion (1);
+				base.SetVersion (version);
 				SnippetMode = true;
+				SetFileType ("document");
 
 				AppendText = new AppendTextCallback (base.AppendText);
 				AppendWord = new AppendTextCallback (base.AppendWord);
@@ -94,6 +97,12 @@ namespace Beagle.Filters {
 
 		public FilterHtml () : this (true) {}
 
+		protected new void SetVersion (int version)
+		{
+			this.version += version;
+			base.SetVersion (this.version);
+		}
+
 		protected bool NodeIsHot (String nodeName) 
 		{
 			return nodeName == "b"
@@ -185,14 +194,16 @@ namespace Beagle.Filters {
 				ignore_level = (ignore_level > 1 ? ignore_level - 1 : 0);
 		}
 
-		protected void HandleContentRichNode (HtmlNode node)
+		protected bool HandleContentRichNode (HtmlNode node)
 		{
 			bool isHot = NodeIsHot (node.Name);
 			bool breaksText = NodeBreaksText (node.Name);
 			bool breaksStructure = NodeBreaksStructure (node.Name);
 
+			bool ret = true;
+
 			if (breaksText)
-				AppendWhiteSpace ();
+				ret = AppendWhiteSpace ();
 
 			if (node.StartTag) {
 				if (isHot) {
@@ -203,8 +214,7 @@ namespace Beagle.Filters {
 					if (attr != String.Empty) {
 						string s = HtmlEntity.DeEntitize (attr);
 						AppendWord (s);
-						AppendWhiteSpace ();
-						added_text_count += (s.Length + 1);
+						ret = AppendWhiteSpace ();
 					}
 				} else if (node.Name == "a") {
 					string attr = node.GetAttributeValue ("href", String.Empty);
@@ -212,39 +222,41 @@ namespace Beagle.Filters {
 						string s = HtmlEntity.DeEntitize (
 							    SW.HttpUtility.UrlDecode (attr, enc));
 						AppendWord (s);
-						AppendWhiteSpace ();
-						added_text_count += (s.Length + 1);
+						ret = AppendWhiteSpace ();
 					}
-				}
+				} else if (node.Name == "br") // both <br> and </br> are used - special case
+					ret = AppendStructuralBreak ();
 			} else { // (! node.StartTag)
 				if (isHot) {
 					HotDown ();
 				}	
 				if (breaksStructure)
-					AppendStructuralBreak ();
+					ret = AppendStructuralBreak ();
 			}
 
 			if (breaksText)
-				AppendWhiteSpace ();
+				ret = AppendWhiteSpace ();
+
+			return ret;
 		}
 
-		int added_text_count = 0;
-
-		protected void HandleTextNode (HtmlNode node)
+		protected bool HandleTextNode (HtmlNode node)
 		{
-			// FIXME Do we need to trim the text ?
+			bool ret = true;
 			String text = ((HtmlTextNode)node).Text;
+
 			if (ignore_level != 0)
-				return; // still ignoring ...
+				return true; // still ignoring ...
 			if (building_text)
 				builder.Append (text);
 			else {
 				string s = HtmlEntity.DeEntitize (text);
-				AppendText (s);
-				added_text_count += s.Length;
+				ret = AppendText (s);
 			}
+
 			//if (hot_stack.Count != 0)
 			//Console.WriteLine (" TEXT:" + text + " ignore=" + ignore_level);
+			return ret;
 		}
 
 		protected bool HandleNodeEventHead (HtmlNode node)
@@ -276,6 +288,7 @@ namespace Beagle.Filters {
 		protected bool HandleNodeEventBody (HtmlNode node)
 		{
 			//Log.Debug ("HandleNodeEventBody (<{0}{1}>)", (node.StartTag ? "" : "/"), node.Name);
+			bool pull = true;
 			switch (node.NodeType) {
 				
 			case HtmlNodeType.Document:
@@ -283,26 +296,25 @@ namespace Beagle.Filters {
 				if (! NodeIsInBody (node.Name)) {
 					break;
 				} else if (! NodeIsContentFree (node.Name)) {
-					HandleContentRichNode (node);
+					pull = HandleContentRichNode (node);
 				} else {
 					HandleContentFreeNode (node);
 				}
 				break;
 				
 			case HtmlNodeType.Text:
-				HandleTextNode (node);
+				pull = HandleTextNode (node);
 				break;
 			}
 
-			if (added_text_count > 2048) {
-				added_text_count = 0;
+			if (! pull)
 				doc.PauseLoad ();
-			}
 
 			return true;
 		}
 
 		// Combined handler when you do not need separate head and body events
+		// No idea of pause and resume implemented here
 		protected bool HandleNodeEvent (HtmlNode node)
 		{
 			switch (node.NodeType) {
@@ -325,10 +337,6 @@ namespace Beagle.Filters {
 				break;
 			}
 
-			// HandleNodeEvent() does not use pause/resume parsing
-			// So, check if more words are allowed to be extracted
-			if (! AllowMoreWords ())
-				return false;
 			return true;
 		}
 
@@ -336,34 +344,30 @@ namespace Beagle.Filters {
 		{
 			enc = null;
 
-			foreach (Property prop in IndexableProperties) {
-				if (prop.Key != StringFu.UnindexedNamespace + "encoding")
-					continue;
+			try {
+				foreach (Property prop in Indexable.Properties) {
+					if (prop.Key != StringFu.UnindexedNamespace + "encoding")
+						continue;
 
-				try {
 					enc = Encoding.GetEncoding ((string) prop.Value);
-				} catch (NotSupportedException) {
-					// Encoding passed in isn't supported.  Maybe
-					// we'll get lucky detecting it from the
-					// document instead.
+					break;
 				}
 
-				break;
-			}
-
-			if (enc == null) {
-				// we need to tell the parser to detect encoding,
-				HtmlDocument temp_doc = new HtmlDocument ();
-				try {
+				if (enc == null) {
+					// we need to tell the parser to detect encoding,
+					HtmlDocument temp_doc = new HtmlDocument ();
 					enc = temp_doc.DetectEncoding (Stream);
-				} catch (NotSupportedException) {
-					enc = Encoding.ASCII;
+					temp_doc = null;
+					Stream.Seek (0, SeekOrigin.Begin);
 				}
-				//Console.WriteLine ("Detected encoding:" + (enc == null ? "null" : enc.EncodingName));
-				temp_doc = null;
-				Stream.Seek (0, SeekOrigin.Begin);
+			} catch (NotSupportedException) {
+				// Encoding passed in isn't supported
 			}
 
+			// Default
+			if (enc == null)
+				enc = Encoding.ASCII;
+
 			doc = new HtmlDocument ();
 			doc.ReportNode += HandleNodeEventHead;
 			doc.StreamMode = true;
@@ -429,7 +433,25 @@ namespace Beagle.Filters {
 		override protected void RegisterSupportedTypes () 
 		{
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("text/html"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/xhtml+xml"));
+		}
+
+		public static TextReader GetHtmlReader (Stream stream, string charset)
+		{
+			if (stream == null)
+				throw new ArgumentNullException ("stream");
+
+			FilterHtml html_filter = new FilterHtml ();
+			html_filter.SnippetMode = false;
+
+			html_filter.Indexable = new Indexable (); // fake an indexable
+			html_filter.AddProperty (Property.NewUnsearched (StringFu.              UnindexedNamespace + "encoding", charset));
+
+			if (! html_filter.Open (stream, false))
+				throw new Exception ("Cannot open html");
+
+			TextReader pr = html_filter.GetTextReader ();
+			return pr;
 		}
 	}
-
 }
