@@ -35,14 +35,19 @@ using Beagle.Util;
 
 namespace Beagle.Filters {
 
+	[PropertyKeywordMapping (Keyword="email", PropertyName="vCard:EMAIL", IsKeyword=false, Description="Email in KAddressbook")]
 	public class FilterKAddressBook : Beagle.Filters.FilterKCal {
 
 		public FilterKAddressBook ()
 		{
-			AddSupportedFlavor (FilterFlavor.NewFromMimeType (ICalParser.KabcMimeType));
+			// 1: Store a prefemail as well, store URL as text.
+			SetVersion (1);
 			SnippetMode = false;
-			if (vCard_property_mapping == null)
-				SetupPropertyMapping ();
+		}
+
+		protected override void RegisterSupportedTypes ()
+		{
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType (ICalParser.KabcMimeType));
 		}
 
 		private static Hashtable vCard_property_mapping = null;
@@ -64,9 +69,9 @@ namespace Beagle.Filters {
 			vCard_property_mapping ["REV"] = new KCalProperty ("dc:date", true, false, KCalType.Date);
 			vCard_property_mapping ["CLASS"] = new KCalProperty ("vCard:CLASS", false, true, KCalType.Text);
 			vCard_property_mapping ["UID"] = new KCalProperty ("vCard:UID", false, true, KCalType.Text);
-			vCard_property_mapping ["EMAIL"] = new KCalProperty ("vCard:EMAIL", false, true, KCalType.Special);
+			vCard_property_mapping ["EMAIL"] = new KCalProperty ("vCard:EMAIL", false, false, KCalType.Special);
 			vCard_property_mapping ["TEL"] = new KCalProperty ("vCard:TEL", false, true, KCalType.Text);
-			vCard_property_mapping ["URL"] = new KCalProperty ("vCard:URL", false, true,KCalType.Text);
+			vCard_property_mapping ["URL"] = new KCalProperty ("vCard:URL", false, false, KCalType.Text);
 		}
 
 		override protected string GetPropertyName (string prop_name, ArrayList paramlist)
@@ -82,6 +87,8 @@ namespace Beagle.Filters {
 							return mapped_prop_name + ":WORK";
 						else if (vcpp.param_value == "HOME")
 							return mapped_prop_name + ":HOME";
+						else if (vcpp.param_value == "CELL")
+							return mapped_prop_name + ":CELL";
 					}
 				}
 				break;
@@ -90,6 +97,22 @@ namespace Beagle.Filters {
 			return mapped_prop_name;
 		}
 
+		private string pref_email;
+		private bool pref_email_set;
+
+		override protected void DoOpen (FileInfo fi)
+		{
+			if (vCard_property_mapping == null)
+				SetupPropertyMapping ();
+		}
+
+		override protected void DoPullProperties ()
+		{
+			pref_email_set = false;
+			base.DoPullProperties ();
+			AddProperty (Beagle.Property.New ("vCard:PREFEMAIL", pref_email));
+		}
+
 		override protected void ProcessPropertySpecial (string prop_name,
 								ArrayList paramlist,
 								string prop_value)
@@ -97,13 +120,15 @@ namespace Beagle.Filters {
 			if (prop_name == "EMAIL") {
 				foreach (KCalPropertyParameter vcpp in paramlist) {
 					if (vcpp.param_name == "TYPE" &&
-					    vcpp.param_value == "PREF")
+					    vcpp.param_value == "PREF") {
 						// Default email
-						AddProperty (Beagle.Property.New (
-							     "vCard:PREFEMAIL",
-							     prop_value));
+						pref_email = prop_value;
+						pref_email_set = true;
+					}
 				}
 				AddProperty (Beagle.Property.New ("vCard:EMAIL", prop_value));
+				if (! pref_email_set)
+					pref_email = prop_value;
 			}
 		}
 	}
