@@ -75,17 +75,19 @@ namespace Beagle.Filters {
 
 		public FilterMPlayerVideo ()
 		{
+			// 1: Version update after FilterVideo was added
+			SetVersion (1);
+
 			PreLoad = false;
+			SetFileType ("video");
 		}
 
+		internal const int Priority = 0; // default
+
 		protected override void RegisterSupportedTypes ()
 		{
-			foreach (string s in mime_types) {
-				FilterFlavor flavor = FilterFlavor.NewFromMimeType (s);
-				flavor.Priority = -1; // Prefer Totem over this one
-
-				AddSupportedFlavor (flavor);
-			}
+			foreach (string s in mime_types)
+				AddSupportedFlavor (FilterFlavor.NewFromMimeType (s));
 		}
 		
 		private string AspectString(float aspect) {
@@ -110,6 +112,12 @@ namespace Beagle.Filters {
 
 		protected override void DoPullProperties ()
 		{
+			if (FileInfo == null) {
+				Log.Error ("FilterMPlayerVideo: Unable to extract properties for non-file data");
+				Error ();
+				return;
+			}
+
 			SafeProcess pc = new SafeProcess ();
 			pc.Arguments = new string [] { "mplayer", "-vo", "null", "-ao", "null", "-frames", "0", "-identify", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
