@@ -41,6 +41,7 @@ namespace Beagle.Filters {
 		public FilterSpreadsheet () 
 		{
 			SnippetMode = true;
+			SetFileType ("document");
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -83,11 +84,8 @@ namespace Beagle.Filters {
 			pc.Arguments = new string [] { "ssindex", "-i", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
 
-			// Runs inside the child process after fork() but before exec()
-			pc.ChildProcessSetup += delegate {
-				// Let ssindex run for 10 seconds, max.
-				SystemPriorities.SetResourceLimit (SystemPriorities.Resource.Cpu, 10);
-			};
+			// Let ssindex run for 10 seconds, max.
+			pc.CpuLimit = 10;
 
 			try {
 				pc.Start ();
