@@ -277,6 +277,14 @@ namespace Beagle.Util {
 			return new FileEnumerable (path, filter, null);
 		}
 
+		static public IEnumerable GetItemNames (string path, FileFilter filter)
+		{
+			FileEnumerable fe;
+			fe = new FileEnumerable (path, filter, null);
+			fe.NamesOnly = true;
+			return fe;
+		}
+
 		static public IEnumerable GetFileInfosRecursive (string path)
 		{
 			foreach (FileInfo i in DirectoryWalker.GetFileInfos (path))
@@ -294,5 +302,19 @@ namespace Beagle.Util {
 		{
 			return GetFileInfosRecursive (dirinfo.FullName);
 		}
+
+		static public int GetNumItems (string path)
+		{
+			int count = 0;
+			FileFilter counting_filter = delegate (string dir, string name) {
+							    count ++;
+							    return false;
+			};
+
+			FileEnumerator dir_enum = new FileEnumerator (path, counting_filter, null);
+			dir_enum.MoveNext ();
+			return count;
+		}
+
 	}
 }
