@@ -65,6 +65,10 @@ namespace Beagle.Daemon {
 					Log.Debug ("Reading or creating attr for {0}", path);
 
 				FileAttributes attr = ifas.Read (path);
+
+				if (attr == null && unique_id == Guid.Empty)
+					unique_id = Guid.NewGuid ();
+
 				// If we pass in a Guid that doesn't match the one we found in the
 				// the attributes, clobber the old attributes and the old unique Guid.
 				if (attr == null
@@ -78,9 +82,6 @@ namespace Beagle.Daemon {
 					attr.UniqueId = unique_id;
 					attr.Path = path;
 					
-					// Now add the new attribute
-					// Note: New attribute should not be added.
-					//ifas.Write (attr);
 					created = true;
 				}
 
@@ -99,7 +100,7 @@ namespace Beagle.Daemon {
 
 		public FileAttributes ReadOrCreate (string path)
 		{
-			return ReadOrCreate (path, Guid.NewGuid ());
+			return ReadOrCreate (path, Guid.Empty);
 		}
 
 		public bool Write (FileAttributes attr)
@@ -161,7 +162,7 @@ namespace Beagle.Daemon {
 			// way that we can be up-to-date.
 			// Also, if attribute has no filter information, try once
 			// again.
-			if (attr == null || ! attr.HasFilterInfo)
+			if (attr == null)
 				return false;
 
 			// Note that when filter is set to null, we ignore
@@ -169,6 +170,9 @@ namespace Beagle.Daemon {
 			// expected behavior...
 			if (filter != null) {
 
+				if (! attr.HasFilterInfo)
+					return false;
+
 				if (attr.FilterName != filter.Name)
 					return false;
 				
@@ -183,6 +187,35 @@ namespace Beagle.Daemon {
 			return IsUpToDate (path, attr);
 		}
 
+		public bool IsUpToDateAndFiltered (string path)
+		{
+			FileAttributes attr;
+
+			attr = Read (path);
+
+			// If there are no attributes set on the file, there is no
+			// way that we can be up-to-date.
+			if (attr == null)
+				return false;
+
+			if (! FilterFactory.DirtyFilterCache)
+				// If the filters are same as in last run,
+				// since attr is not null, check the timestamps (bypass HasFilterInfo)
+				return IsUpToDate (path, attr);
+
+			// If there is a new filter in the mean time
+			// take previous filter information into consideration.
+			if (! attr.HasFilterInfo)
+				return false;
+
+			int current_filter_version;
+			current_filter_version = FilterFactory.GetFilterVersion (attr.FilterName);
+			if (current_filter_version > attr.FilterVersion)
+				return false;
+
+			return IsUpToDate (path, attr);
+		}
+
 		public bool IsUpToDate (string path)
 		{
 			return IsUpToDate (path, (Filter) null);
