@@ -39,11 +39,13 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 		private object big_lock = new object ();
 		private bool is_active = false;
+		private FileSystemQueryable queryable;
 		private Handler handler;
 		private Queue to_be_crawled = new Queue ();
 
-		public TreeCrawlTask (Handler handler)
+		public TreeCrawlTask (FileSystemQueryable queryable, Handler handler)
 		{
+			this.queryable = queryable;
 			this.handler = handler;
 			this.Tag = "Tree Crawler";
 			this.Priority = Scheduler.Priority.Delayed;
@@ -53,6 +55,14 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			get { lock (big_lock) return is_active; }
 		}
 
+		// Must be called from inside big_lock
+		private void SetIsActive (bool is_active, DirectoryModel current_dir)
+		{
+			this.is_active = is_active;
+
+			queryable.UpdateIsIndexing (current_dir);
+		}
+
 		// Returns 'true' if the queue was empty before adding
 		// this item.
 		public bool Add (DirectoryModel dir)
@@ -80,8 +90,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 					return;
 				}
 				dir = to_be_crawled.Dequeue () as DirectoryModel;
-				Log.Debug ("Running tree crawl task");
-				is_active = true;
+
+				if (FileSystemQueryable.Debug)
+					Log.Debug ("Running tree crawl task");
+
+				SetIsActive (true, dir);
 			}
 			
 			LuceneQueryable queryable = (LuceneQueryable) Source;
@@ -113,7 +126,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		private void DoneCrawling ()
 		{
 			Log.Debug ("Done crawling directory tree!!!");
-			is_active = false;
+			SetIsActive (false, null);
 		}
 
 	}
