@@ -35,13 +35,15 @@ namespace Beagle.Daemon {
 
 		void Start ();
 
+		// Should not modify query
 		bool AcceptQuery (Query query);
 
+		// Should not modify query
 		void DoQuery (Query query,
 			      IQueryResult result,
 			      IQueryableChangeData data);
 
-		string GetSnippet (string[] query_terms, Hit hit);
+		ISnippetReader GetSnippet (string[] query_terms, Hit hit, bool full_text);
 
 		QueryableStatus GetQueryableStatus ();
 	}
