@@ -27,6 +27,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 
 using Beagle.Util;
@@ -62,7 +63,18 @@ namespace Beagle.Daemon.KMailQueryable {
 		public KMailQueryable Queryable {
 		    get { return queryable; }
 		}
-		
+
+		////////////// Progress tracking //////////////////
+
+		private double progress = 0;
+
+		internal double Progress {
+			set { progress = value; }
+			get { return progress; }
+		}
+
+		///////////////////////////////////////////////////
+
 		private string lastGoodDirPath = ""; // cache last successful directory
 
 		public KMailIndexer (KMailQueryable queryable, string account, string root)
@@ -71,15 +83,16 @@ namespace Beagle.Daemon.KMailQueryable {
 			account_name = account;
 			mail_root = root;
 			mail_directories = new ArrayList ();
-			Logger.Log.Debug ("mail_directories created for:" + mail_root + " (" + mail_directories.Count + ")");
 			folder_directories = new ArrayList ();
 			mbox_files = new ArrayList ();
 
 			excludes = new ArrayList ();
-			excludes.Add ("spam");
-			excludes.Add ("outbox");
-			excludes.Add ("trash");
-			excludes.Add ("drafts");
+			List<string[]> values = Conf.Daemon.GetListOptionValues (Conf.Names.ExcludeMailfolder);
+			if (values == null)
+				return;
+
+			foreach (string[] item in values)
+				excludes.Add (item [0].ToLower ());
 		}
 
 		/**
@@ -91,13 +104,6 @@ namespace Beagle.Daemon.KMailQueryable {
 					     string srcpath,
 					     Inotify.EventType type)
 		{
-			//FIXME this case should NEVER occur, still it does
-			if (mail_directories == null) {
-				Logger.Log.Debug ("*** WEIRD AVIRAM CASE for :" + mail_root);
-				Logger.Log.Debug ("Received inotify event{3} for {4}: path={0}, subitem={1}, srcpath={2}", path, subitem, srcpath, type, mail_root);
-				return;
-			}
-			
 			if (subitem == "")
 				return;
 			string fullPath = Path.Combine (path, subitem);
@@ -368,7 +374,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			Indexable indexable = new Indexable (file_uri);
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
-			indexable.CacheContent = false;
+			indexable.CacheContent = true;
 			indexable.Crawled = crawl;
 
 			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "kmail"));
@@ -394,7 +400,7 @@ namespace Beagle.Daemon.KMailQueryable {
 			indexable.Timestamp = message.Date.ToUniversalTime ();
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
-			indexable.CacheContent = false;
+			indexable.CacheContent = true;
 			indexable.Crawled = true; // mbox files will be stored in temp files, so flush them when done
 
 			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "kmail"));
