@@ -162,52 +162,43 @@ namespace Beagle.Daemon.KMailQueryable {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-		override public string GetSnippet (string[] query_terms, Hit hit)
-		{
-			Log.Debug ("Fetching snippet for " + hit.Uri.LocalPath);
-			// FIXME: Also handle mbox emails
-			if (! hit.Uri.IsFile)
-				return null;
-
-			// Dont get snippets from attachments, they arent even indexed currently
-			if (hit.ParentUri != null)
-				return null;
-			
-			int mail_fd = Mono.Unix.Native.Syscall.open (hit.Uri.LocalPath, Mono.Unix.Native.OpenFlags.O_RDONLY);
-			if (mail_fd == -1)
-				return null;
-
-			InitializeGMime ();
-			GMime.StreamFs stream = new GMime.StreamFs (mail_fd);
-			GMime.Parser parser = new GMime.Parser (stream);
-			GMime.Message message = parser.ConstructMessage ();
-			stream.Dispose ();
-			parser.Dispose ();
-
-			bool html = false;
-			string body = message.GetBody (true, out html);
-			// FIXME: Also handle snippets from html message parts - involves invoking html filter
-			if (html) {
-				Log.Debug ("No text/plain message part in " + hit.Uri);
-				message.Dispose ();
-				return null;
-			}
-
-			StringReader reader = new StringReader (body);
-			string snippet = SnippetFu.GetSnippet (query_terms, reader);
-			message.Dispose ();
-
-			return snippet;
-		}
-
-		/////////////////////////////////////////////////////////////////////////////
-
 		// FIXME: How to determine if an mbox hit is valid without scanning the whole file
 
 		public string Name {
 			get { return "KMail"; }
 		}
 
+		protected override int ProgressPercent {
+			get {
+				if (! IsIndexing)
+					return -1;
+
+				double local_progress = 0.5;
+				if (local_indexer != null)
+					local_progress = local_indexer.Progress/2;
+				
+				double dimap_progress = 0.5;
+				if (dimap_indexer != null)
+					dimap_progress = dimap_indexer.Progress/2;
+
+				return (int) (local_progress * 100 + dimap_progress * 100);
+			}
+		}
+
+		private int indexer_indexing = 0;
+		internal bool Indexing {
+			set {
+				// FIXME: This only works because both indexers call
+				// Indexing=true exactly once and Indexing=false exactly once.
+				indexer_indexing += (value ? 1 : -1);
+
+				if (indexer_indexing == 0)
+					IsIndexing = false;
+				else
+					IsIndexing = true;
+			}
+		}
+
 		/** 
 		 * path of local maildir - mine is in ~/.Mail
 		 * This is distribution specific. Mandrake puts kmail mails in
