@@ -25,10 +25,11 @@
 //
 
 using System;
-using System.Collections;
 using System.IO;
+using System.Xml;
 using System.Text;
 using System.Threading;
+using System.Collections;
 
 using Beagle.Daemon;
 using Beagle.Util;
@@ -69,10 +70,8 @@ namespace Beagle.Daemon.KopeteQueryable {
 			if (Inotify.Enabled)
 				Watch (log_dir);
 
-			State = QueryableState.Crawling;
 			crawler = new KopeteCrawler (log_dir);
 			Crawl ();
-			State = QueryableState.Idle;
 
 			if (!Inotify.Enabled) {
 				Scheduler.Task task = Scheduler.TaskFromHook (new Scheduler.TaskHook (CrawlHook));
@@ -179,7 +178,7 @@ namespace Beagle.Daemon.KopeteQueryable {
 			Indexable indexable = new Indexable (uri);
 			indexable.ContentUri = uri;
 			indexable.Timestamp = File.GetLastWriteTimeUtc (filename);
-			indexable.MimeType = KopeteLog.MimeType;
+			indexable.MimeType = "beagle/x-kopete-log";
 			indexable.HitType = "IMLog";
 			indexable.CacheContent = false;
 
@@ -201,18 +200,17 @@ namespace Beagle.Daemon.KopeteQueryable {
 			ThisScheduler.Add (task);
 		}
 
-		override protected double RelevancyMultiplier (Hit hit)
+		protected override double RelevancyMultiplier (Hit hit)
 		{
-			return HalfLifeMultiplierFromProperty (hit, 0.25,
-							       "fixme:endtime", "fixme:starttime");
+			return HalfLifeMultiplierFromProperty (hit, 0.25, "fixme:endtime", "fixme:starttime");
 		}
 
-		override protected bool HitFilter (Hit hit) 
+		protected override bool HitFilter (Hit hit) 
 		{
 			ImBuddy buddy = list.Search (hit ["fixme:speakingto"]);
 			
 			if (buddy != null) {
-				if (buddy.Alias != "")
+				if (!String.IsNullOrEmpty (buddy.Alias))
 					hit ["fixme:speakingto_alias"] = buddy.Alias;
 
 				// FIXME: Icons?
@@ -220,5 +218,60 @@ namespace Beagle.Daemon.KopeteQueryable {
 			
 			return true;
 		}
+
+		public override ISnippetReader GetSnippet (string [] query_terms, Hit hit, bool full_text)
+		{
+			TextReader reader = TextCache.UserCache.GetReader (hit.Uri);
+
+			if (reader == null)
+				return null;
+
+			KopeteSnippetReader snippet_reader = new KopeteSnippetReader (reader);
+
+			return SnippetFu.GetSnippet (query_terms, snippet_reader, full_text);
+		}
+
+		private class KopeteSnippetReader : TextReader {
+			
+			private StringBuilder sb;
+			private XmlTextReader reader;
+			
+			public KopeteSnippetReader (TextReader reader)
+			{
+				this.sb = new StringBuilder ();
+				this.reader = new XmlTextReader (reader);
+			}
+			
+			public override string ReadLine ()
+			{				
+				while (reader.Read ()) {
+					if (reader.NodeType != XmlNodeType.Element && reader.Name != "msg")
+						continue;
+				
+					sb.Length = 0;
+
+					if (!String.IsNullOrEmpty (reader ["nick"]))
+						sb.Append (reader["nick"]);
+					else
+						sb.Append (reader ["from"]);
+
+					sb.Append (": ");
+
+					// Advance to the text node for the actual message
+					reader.Read ();
+				
+					sb.Append (reader.Value);
+
+					return sb.ToString ();
+				}
+
+				return null;
+			}
+
+			public override void Close ()
+			{
+				reader.Close ();
+			}
+		}
 	}
 }
