@@ -157,6 +157,11 @@ namespace Lucene.Net.Index
 			return in_Renamed.Document(n);
 		}
 		
+		public override Document Document(int n, string[] fields)
+		{
+			return in_Renamed.Document(n, fields);
+		}
+		
 		public override bool IsDeleted(int n)
 		{
 			return in_Renamed.IsDeleted(n);
@@ -245,4 +250,4 @@ namespace Lucene.Net.Index
 			return in_Renamed.GetFieldNames(fieldNames);
 		}
 	}
-}
\ No newline at end of file
+}
