@@ -13,8 +13,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+
 using System;
 using Directory = Lucene.Net.Store.Directory;
+
 namespace Lucene.Net.Index
 {
 	
@@ -23,23 +25,23 @@ namespace Lucene.Net.Index
 	/// set.  
 	/// </summary>
 	
-	sealed public class TermInfosReader
+	public sealed class TermInfosReader
 	{
 		private Directory directory;
 		private System.String segment;
 		private FieldInfos fieldInfos;
-
-		private SegmentTermEnum termEnum;
+		
+		private System.LocalDataStoreSlot enumerators = System.Threading.Thread.AllocateDataSlot();
 		private SegmentTermEnum origEnum;
 		private long size;
 		
-        private Term[] indexTerms = null;
-        private TermInfo[] indexInfos;
-        private long[] indexPointers;
+		private Term[] indexTerms = null;
+		private TermInfo[] indexInfos;
+		private long[] indexPointers;
 		
-        private SegmentTermEnum indexEnum;
-
-        public /*internal*/ TermInfosReader(Directory dir, System.String seg, FieldInfos fis)
+		private SegmentTermEnum indexEnum;
+		
+		public /*internal*/ TermInfosReader(Directory dir, System.String seg, FieldInfos fis)
 		{
 			directory = dir;
 			segment = seg;
@@ -48,10 +50,18 @@ namespace Lucene.Net.Index
 			origEnum = new SegmentTermEnum(directory.OpenInput(segment + ".tis"), fieldInfos, false);
 			size = origEnum.size;
 			
-            indexEnum = new SegmentTermEnum(directory.OpenInput(segment + ".tii"), fieldInfos, true);
+			indexEnum = new SegmentTermEnum(directory.OpenInput(segment + ".tii"), fieldInfos, true);
 		}
-				
-        public int GetSkipInterval()
+		
+        /*  Leaving this here will cause a memory leak under .NET 1.1
+        ~TermInfosReader()
+		{
+            // patch for pre-1.4.2 JVMs, whose ThreadLocals leak
+            //System.Threading.Thread.SetData(enumerators, null);
+		}
+        */
+		
+		public int GetSkipInterval()
 		{
 			return origEnum.skipInterval;
 		}
@@ -60,9 +70,9 @@ namespace Lucene.Net.Index
 		{
 			if (origEnum != null)
 				origEnum.Close();
-            if (indexEnum != null)
-                indexEnum.Close();
-        }
+			if (indexEnum != null)
+				indexEnum.Close();
+		}
 		
 		/// <summary>Returns the number of term/value pairs in the set. </summary>
 		internal long Size()
@@ -72,42 +82,44 @@ namespace Lucene.Net.Index
 		
 		private SegmentTermEnum GetEnum()
 		{
+			SegmentTermEnum termEnum = (SegmentTermEnum) System.Threading.Thread.GetData(enumerators);
 			if (termEnum == null)
 			{
 				termEnum = Terms();
+				System.Threading.Thread.SetData(enumerators, termEnum);
 			}
 			return termEnum;
 		}
 		
-        private void  EnsureIndexIsRead()
-        {
-            lock (this)
-            {
-                if (indexTerms != null)
-                // index already read
-                    return ; // do nothing
-                try
-                {
-                    int indexSize = (int) indexEnum.size; // otherwise read index
+		private void  EnsureIndexIsRead()
+		{
+			lock (this)
+			{
+				if (indexTerms != null)
+				// index already read
+					return ; // do nothing
+				try
+				{
+					int indexSize = (int) indexEnum.size; // otherwise read index
 					
-                    indexTerms = new Term[indexSize];
-                    indexInfos = new TermInfo[indexSize];
-                    indexPointers = new long[indexSize];
+					indexTerms = new Term[indexSize];
+					indexInfos = new TermInfo[indexSize];
+					indexPointers = new long[indexSize];
 					
-                    for (int i = 0; indexEnum.Next(); i++)
-                    {
-                        indexTerms[i] = indexEnum.Term();
-                        indexInfos[i] = indexEnum.TermInfo();
-                        indexPointers[i] = indexEnum.indexPointer;
-                    }
-                }
-                finally
-                {
-                    indexEnum.Close();
-                    indexEnum = null;
-                }
-            }
-        }
+					for (int i = 0; indexEnum.Next(); i++)
+					{
+						indexTerms[i] = indexEnum.Term();
+						indexInfos[i] = indexEnum.TermInfo();
+						indexPointers[i] = indexEnum.indexPointer;
+					}
+				}
+				finally
+				{
+					indexEnum.Close();
+					indexEnum = null;
+				}
+			}
+		}
 		
 		/// <summary>Returns the offset of the greatest index entry which is less than or equal to term.</summary>
 		private int GetIndexOffset(Term term)
@@ -140,9 +152,9 @@ namespace Lucene.Net.Index
 			if (size == 0)
 				return null;
 			
-            EnsureIndexIsRead();
+			EnsureIndexIsRead();
 			
-            // optimize sequential access: first try scanning cached enum w/o seeking
+			// optimize sequential access: first try scanning cached enum w/o seeking
 			SegmentTermEnum enumerator = GetEnum();
 			if (enumerator.Term() != null && ((enumerator.Prev() != null && term.CompareTo(enumerator.Prev()) > 0) || term.CompareTo(enumerator.Term()) >= 0))
 			{
@@ -159,13 +171,13 @@ namespace Lucene.Net.Index
 		/// <summary>Scans within block for matching term. </summary>
 		private TermInfo ScanEnum(Term term)
 		{
-            SegmentTermEnum enumerator = GetEnum();
-            enumerator.ScanTo(term);
-            if (enumerator.Term() != null && term.CompareTo(enumerator.Term()) == 0)
-                return enumerator.TermInfo();
-            else
-                return null;
-        }
+			SegmentTermEnum enumerator = GetEnum();
+			enumerator.ScanTo(term);
+			if (enumerator.Term() != null && term.CompareTo(enumerator.Term()) == 0)
+				return enumerator.TermInfo();
+			else
+				return null;
+		}
 		
 		/// <summary>Returns the nth term in the set. </summary>
 		internal Term Get(int position)
@@ -197,8 +209,8 @@ namespace Lucene.Net.Index
 			if (size == 0)
 				return - 1;
 			
-            EnsureIndexIsRead();
-            int indexOffset = GetIndexOffset(term);
+			EnsureIndexIsRead();
+			int indexOffset = GetIndexOffset(term);
 			SeekEnum(indexOffset);
 			
 			SegmentTermEnum enumerator = GetEnum();
