@@ -32,7 +32,7 @@ using Beagle.Util;
 
 namespace Beagle.Daemon {
 
-	[RequestMessage (typeof (Query))]
+	[RequestMessage (typeof (Beagle.Query))]
 	public class QueryExecutor : RequestMessageExecutor {
 
 		private Query query;
@@ -40,12 +40,16 @@ namespace Beagle.Daemon {
 
 		private void DisconnectResult ()
 		{
+			if (this.result == null)
+				return;
+
 			this.result.HitsAddedEvent -= OnResultHitsAdded;
 			this.result.HitsSubtractedEvent -= OnResultHitsSubtracted;
 			this.result.FinishedEvent -= OnResultFinished;
 					
 			this.result.Cancel ();
 			this.result.Dispose ();
+			this.result = null;
 		}
 
 		private void AttachResult ()
@@ -55,9 +59,9 @@ namespace Beagle.Daemon {
 			this.result.FinishedEvent += OnResultFinished;
 		}
 
-		public void OnResultHitsAdded (QueryResult result, ICollection some_hits)
+		public void OnResultHitsAdded (QueryResult result, ICollection some_hits, int total_results)
 		{
-			HitsAddedResponse response = new HitsAddedResponse (some_hits);
+			HitsAddedResponse response = new HitsAddedResponse (some_hits, total_results);
 
 			this.SendAsyncResponse (response);
 		}
@@ -101,6 +105,8 @@ namespace Beagle.Daemon {
 		{
 			QueryDriver.ChangedEvent -= OnQueryDriverChanged;
 			DisconnectResult ();
+
+			this.query = null;
 		}
 	}
 }
