@@ -427,7 +427,10 @@ namespace Beagle.Daemon {
 			if (this.executor != null) {
 				this.executor.Cleanup ();
 				this.executor.AsyncResponseEvent -= OnAsyncResponse;
+				this.executor = null;
 			}
+
+			Server.RunGC ();
 		}
 
 		public void WatchCallback (IAsyncResult ar)
@@ -632,10 +635,19 @@ namespace Beagle.Daemon {
 
 		public static Hashtable item_handlers = new Hashtable ();
 
+		private static System.Timers.Timer gc_timer = null;
+
 		static Server ()
 		{
 			foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies ())
 				ScanAssemblyForExecutors (assembly);
+
+			if (Environment.GetEnvironmentVariable ("BEAGLE_FORCE_GC") != null) {
+				gc_timer = new System.Timers.Timer ();
+				gc_timer.Interval = 60000; // GC Collect to run after xxx msecs of last client closing
+				gc_timer.Elapsed += new ElapsedEventHandler (GCTimerElapsed);
+				gc_timer.AutoReset = false;
+			}
 		}
 
 		public Server (string name, bool indexhelper, bool enable_network_svc)
@@ -676,6 +688,32 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		static int gc_count = 0;
+
+		static internal void RunGC ()
+		{
+			if (gc_timer == null)
+				return;
+
+			gc_timer.Stop ();
+			gc_timer.AutoReset = true;
+			gc_count = 5;
+			gc_timer.Start ();
+		}
+
+		private static void GCTimerElapsed (object sender, ElapsedEventArgs e)
+    		{
+			Console.WriteLine("After GC collection: {0} (RSS {1})", GC.GetTotalMemory (false), SystemInformation.VmRss);
+
+			GC.Collect (2); // The argument is a no-op in mono
+			GC.WaitForPendingFinalizers();
+
+			if (--gc_count == 0) {
+				gc_timer.AutoReset = false;
+				gc_timer.Stop ();
+			}
+    		}
+
 		private static void OnShutdown ()
 		{
 			lock (live_handlers) {
@@ -688,7 +726,6 @@ namespace Beagle.Daemon {
 
 		private void Run ()
 		{
-			this.running = true;
 			this.unix_listener.Start ();
 
 			if (! Shutdown.WorkerStart (this, String.Format ("server '{0}'", socket_path)))
@@ -729,7 +766,7 @@ namespace Beagle.Daemon {
 
 			Logger.Log.Debug ("Server '{0}' shut down", this.socket_path);
 		}
-		
+
 		private void HttpRun ()
 		{
 			http_listener = new HttpListener ();
@@ -860,6 +897,17 @@ namespace Beagle.Daemon {
 			http_listener = null;
 		}
 
+		private void RunInThread ()
+		{
+			try {
+				this.Run ();
+			} catch (ThreadAbortException) {
+				Thread.ResetAbort ();
+				Log.Debug ("Breaking out of UnixListener -- shutdown requested");
+				Shutdown.WorkerFinished (this);
+			}
+		}
+
 		public void Start ()
 		{
 			if (!initialized)
@@ -868,6 +916,8 @@ namespace Beagle.Daemon {
 			if (Shutdown.ShutdownRequested)
 				return;
 
+			this.running = true;
+
 			if (! indexhelper) {
 				Config config = Conf.Get (Conf.Names.NetworkingConfig);
 				webinterface = config.GetOption ("WebInterface", false);
@@ -878,15 +928,7 @@ namespace Beagle.Daemon {
 					ExceptionHandlingThread.Start (new ThreadStart (this.HttpRun));
 			}
 
-			ExceptionHandlingThread.Start (new ThreadStart (delegate () {
-							    try {
-								    this.Run ();
-							    } catch (ThreadAbortException) {
-								    Thread.ResetAbort ();
-							    	    Log.Debug ("Breaking out of UnixListener -- shutdown requested");
-							    	    Shutdown.WorkerFinished (this);
-							    }
-						    }));
+			ExceptionHandlingThread.Start (new ThreadStart (this.RunInThread));
 		}
 
 		private void StartWebserver ()
@@ -899,6 +941,9 @@ namespace Beagle.Daemon {
 
 		public void Stop ()
 		{
+			if (gc_timer != null)
+				gc_timer.Stop ();
+
 			if (this.running) {
 				this.running = false;
 				this.unix_listener.Stop ();
