@@ -25,15 +25,18 @@ namespace Search.Tiles {
 
 			// FIXME: Show album art if any. Needs implementation in Beagle.Util
 
-			string title = Hit.GetFirstProperty ("fixme:title");
-			if (title != null && title != "")
+			string title = Hit.GetFirstProperty ("dc:title");
+			if (String.IsNullOrEmpty (title))
+				title = Hit.GetFirstProperty ("fixme:title");
+
+			if (! String.IsNullOrEmpty (title))
 				Title = title;
 
 			string artist = Hit.GetFirstProperty ("fixme:artist");
-			if (artist != null && artist != "")
+			if (! String.IsNullOrEmpty (artist))
 				Description = artist;
 
-//			AddAction (new TileAction (Catalog.GetString ("Add to Library"), AddToLibrary));
+			//AddAction (new TileAction (Catalog.GetString ("Add to Library"), AddToLibrary));
 		}
 
 		// FIXME: Check if Banshee exists and supports this?
