@@ -73,8 +73,6 @@ namespace Search.Tiles {
 			if (home_phone != null && home_phone != "")
 				details.AddLabelPair (Catalog.GetString ("Home Phone:"), home_phone);
 			
-			details.AddFinalLine ();
-
 			return details;
 		}
 
@@ -82,7 +80,7 @@ namespace Search.Tiles {
 		{
 			SafeProcess p = null;
 
-   			if (client == "evolution") {
+   			if (client == "evolution" || (client == null && uri.StartsWith ("contacts:"))) {
 				p = new SafeProcess ();
 				p.Arguments = new string [2];
 				p.Arguments [0] = "evolution";
@@ -102,8 +100,13 @@ namespace Search.Tiles {
 
 		public override void Open ()
 		{
-			SafeProcess p = GetClientProcess (Hit.GetFirstProperty ("fixme:client"), Hit.Uri.ToString ());
+			SafeProcess p = GetClientProcess (Hit.GetFirstProperty ("fixme:client"), Hit.EscapedUri);
 
+			if (p == null) {
+				Console.WriteLine ("Opening contact '{0}' is unsupported!", Hit.EscapedUri);
+				return;
+			}
+			
 			try {
 				p.Start ();
 			} catch (SafeProcessException e) {
