@@ -27,7 +27,7 @@ namespace Search.Tiles {
 
 			Title = Hit ["beagle:ExactFilename"];
 
-			if (Hit ["beagle:FilenameExtension"].Length > 0)
+			if (Hit ["beagle:FilenameExtension"] != null && Hit ["beagle:FilenameExtension"].Length > 0)
 				Description = Hit ["beagle:FilenameExtension"].Substring (1).ToUpper ();
 			
 			if (Hit ["fixme:width"] != null && Hit ["fixme:width"] != "")
@@ -45,8 +45,8 @@ namespace Search.Tiles {
 
 			// Draw the F-Spot overlay
 			if (size > 32 && Hit ["fspot:IsIndexed"] == "true") {
-				Gdk.Pixbuf icon = image.Pixbuf;
-				Gdk.Pixbuf emblem = Beagle.Images.GetPixbuf ("emblem-fspot.png", 24, 24);
+				Gdk.Pixbuf emblem = WidgetFu.LoadThemeIcon ("f-spot", 24);
+				Gdk.Pixbuf icon = image.Pixbuf.Copy ();
 
 				if (icon == null || emblem == null)
 					return;
@@ -54,13 +54,17 @@ namespace Search.Tiles {
 				// FIXME: Ideally we'd composite into a fresh new pixbuf of
 				// the correct size in this case, but really, who's going to
 				// have images shorter or narrower than 16 pixels in f-spot??
-				if (icon.Height < emblem.Height || icon.Width < emblem.Width)
+				if (icon.Height < emblem.Height || icon.Width < emblem.Width) {
+					icon.Dispose ();
+					emblem.Dispose ();
 					return;
+				}
 
 				emblem.Composite (icon, 0,  icon.Height - emblem.Height, emblem.Width,
 						  emblem.Height, 0,  icon.Height - emblem.Height, 1,  1,
 						  Gdk.InterpType.Bilinear, 255);
 
+				image.Pixbuf.Dispose ();
 				image.Pixbuf = icon;
 			}
 		}
@@ -75,9 +79,25 @@ namespace Search.Tiles {
 			details.AddTextLabel (Description);
 			details.AddNewLine ();
 
+			string[] tags = Hit.GetProperties ("dc:subject");
+			if (tags != null && tags.Length > 0)
+				details.AddLabelPair (Catalog.GetString ("Tags:"), String.Join (", ", tags));
+
 			details.AddLabelPair (Catalog.GetString ("Modified:"), Utils.NiceVeryLongDate (Hit.FileInfo.LastWriteTime));
 			details.AddLabelPair (Catalog.GetString ("Full Path:"), Hit.Uri.LocalPath);
 
+			// Get comments from the image.  FIXME: These should be unified into a single field.
+			string comment = Hit.GetFirstProperty ("png:comment");
+
+			if (String.IsNullOrEmpty (comment))
+				comment = Hit.GetFirstProperty ("jfif:Comment");
+
+			if (! String.IsNullOrEmpty (comment)) {
+				details.AddLabelPair (Catalog.GetString ("Comment:"), comment);
+				//details.AddNewLine ();
+				//details.AddTextLabel (comment);
+			}
+
 			if (Hit ["fspot:Description"] != null && Hit ["fspot:Description"] != "") {
 				details.AddNewLine ();
 				details.AddTextLabel (Hit ["fspot:Description"]);
