@@ -66,6 +66,9 @@ namespace Search.Tiles {
 
 			if (Utils.GetFirstPropertyOfParent (Hit, "fixme:client") == "evolution")
 				AddAction (new TileAction (Catalog.GetString ("Send in Mail"), SendInMail));
+			if ((Utils.GetFirstPropertyOfParent (hit, "fixme:isSent") == null)){
+				AddAction ( new TileAction (Catalog.GetString ("Find Messages From Sender"), Gtk.Stock.Find, FindAllFromSender));
+			}
 		}
 
 		protected override void LoadIcon (Gtk.Image image, int size)
@@ -109,42 +112,40 @@ namespace Search.Tiles {
 
 			if (folder != null)
 				details.AddLabelPair (Catalog.GetString ("Folder:"), folder);
-
-			// Mail messages don't save snippets yet.
-			//details.AddSnippet ();
-
+			
+			details.AddSnippet ();
+			
 			return details;
 		}
 
-		public static SafeProcess GetClientProcess (string client, string uri)
+		public static SafeProcess GetClientProcess (Beagle.Hit hit)
 		{
+			string client = null;
 			SafeProcess p = null;
-
+			
+			if (hit.ParentUri != null)
+				client = Utils.GetFirstPropertyOfParent (hit, "fixme:client");
+			else
+				client = hit.GetFirstProperty ("fixme:client");
+			
 			if (client == "evolution") {
 				p = new SafeProcess ();
 				p.Arguments = new string [2];
 				p.Arguments [0] = "evolution";
-				p.Arguments [1] = uri;
-#if ENABLE_THUNDERBIRD
-			} else if (client == "thunderbird") {
-				p = new SafeProcess ();
-				p.Arguments = new string [3];
-				p.Arguments [0] = Thunderbird.ExecutableName;
-				p.Arguments [1] = "-mail";
-				p.Arguments [2] = uri;
-#endif
+				p.Arguments [1] = (hit.ParentUri != null ? hit.EscapedParentUri : hit.EscapedUri);
 			}
+#if ENABLE_THUNDERBIRD
+			else if (client == "thunderbird")  {
+			     p = Thunderbird.GetSafeProcess ("-viewbeagle", hit.GetFirstProperty ("fixme:uri"));
+			}
+#endif
 
 			return p;
 		}
 
 		public override void Open ()
 		{
-			SafeProcess p;
-			if (Hit.ParentUri != null) 
-				p = GetClientProcess (Utils.GetFirstPropertyOfParent (Hit, "fixme:client"), Hit.EscapedParentUri);
-			else
-				p = GetClientProcess (Utils.GetFirstPropertyOfParent (Hit, "fixme:client"), Hit.EscapedUri);
+			SafeProcess p = GetClientProcess (Hit);
 			
 			if (p == null) {
 				OpenFromMime (Hit);
@@ -176,5 +177,16 @@ namespace Search.Tiles {
 				Console.WriteLine ("Error launching Evolution composer: " + e.Message);
 			}
 		}
+		public void FindAllFromSender()
+		{
+			SafeProcess p = new SafeProcess ();
+			string addr = Hit.GetFirstProperty("fixme:from_address");
+			p.Arguments = new string [] { "beagle-search", String.Format ("mailfromaddr:{0}", addr) };
+			try {
+				p.Start () ;
+			} catch (Exception e) {
+				Console.WriteLine ("Error launching new search: " + e.Message);
+			}
+		}
 	}
 }
