@@ -3,6 +3,7 @@ using System.IO;
 using System.Diagnostics;
 using System.Collections;
 using Mono.Unix;
+using Beagle.Util;
 
 namespace Search.Tiles {
 
@@ -31,14 +32,24 @@ namespace Search.Tiles {
 
 		protected override void LoadIcon (Gtk.Image image, int size)
 		{
+			Gdk.Pixbuf pixbuf = null;
+
 			string path = Hit ["fixme:cachedimg"];
 			if (path != null && File.Exists (path)) {
-				image.Pixbuf = new Gdk.Pixbuf (path);
+				try {
+					pixbuf = new Gdk.Pixbuf (path);
+				} catch (GLib.GException) {
+					// Catch in case of an invalid pixbuf.
+				}
+			}
 
-				if (image.Pixbuf.Width > size || image.Pixbuf.Height > size)
-					image.Pixbuf = image.Pixbuf.ScaleSimple (size, size, Gdk.InterpType.Bilinear);
-			} else
-				image.Pixbuf = WidgetFu.LoadThemeIcon ("gnome-fs-bookmark", size); // FIXME: RSS icon?
+			if (pixbuf != null && (pixbuf.Width > size || pixbuf.Height > size))
+				pixbuf = pixbuf.ScaleSimple (size, size, Gdk.InterpType.Bilinear);
+
+			if (pixbuf == null)
+				pixbuf = WidgetFu.LoadThemeIcon ("gnome-fs-bookmark", size); // FIXME: RSS icon?
+
+			image.Pixbuf = pixbuf;
 		}
 
 		protected override DetailsPane GetDetails ()
@@ -55,7 +66,22 @@ namespace Search.Tiles {
 
 		public override void Open ()
 		{
-			base.OpenFromUri (Hit ["dc:identifier"]);
+			// If we are not a feed from Thunderbird just open based on mime
+			if (Hit.GetFirstProperty ("fixme:client") != "thunderbird") {
+				base.OpenFromUri (Hit ["dc:identifier"]);
+				return;
+			}
+
+#if ENABLE_THUNDERBIRD			
+			// Here's the Thunderbird specific part
+			SafeProcess p = Thunderbird.GetSafeProcess ("-viewbeagle", Hit.GetFirstProperty ("fixme:uri"));
+			
+			try {
+				p.Start ();
+			} catch (SafeProcessException e) {
+				Console.WriteLine ("Unable to run {0}: {1}", p.Arguments [0], e.Message);
+			}
+#endif
 		}
 	}
 }
