@@ -181,7 +181,9 @@ namespace Search.Tiles {
 
 #if ENABLE_OPEN_WITH
 			if (EnableOpenWith) {
-				OpenWithMenu owm = new OpenWithMenu (Hit ["beagle:MimeType"]);
+				// FIXME: Not sure if going with the parent is
+				// the right thing to do in all cases.
+				OpenWithMenu owm = new OpenWithMenu (Utils.GetFirstPropertyOfParent (hit, "beagle:MimeType"));
 				owm.ApplicationActivated += OpenWith;
 				owm.AppendToMenu (menu);
 			}
@@ -272,15 +274,16 @@ namespace Search.Tiles {
 			// other tags escaped.
 
 			// FIXME: hacky, fix the snippeting in the daemon
+			
 			snippet = GLib.Markup.EscapeText (((SnippetResponse)response).Snippet);
 			snippet = Regex.Replace (snippet, "&lt;font color=&quot;.*?&quot;&gt;&lt;b&gt;(.*?)&lt;/b&gt;&lt;/font&gt;", "<b>$1</b>");
-
-			EmitGotSnippet ();
+			if(snippet.Trim().Length > 0)
+				EmitGotSnippet ();
 		}
 
 		private void EmitGotSnippet ()
 		{
-			if (snippet != null && snippet != "" && GotSnippet != null)
+			if (!String.IsNullOrEmpty (snippet) && GotSnippet != null)
 				GotSnippet (snippet);
 		}
 
@@ -332,30 +335,42 @@ namespace Search.Tiles {
 			string command = null, item;
 			bool expects_uris = false;
 
+			string mimetype = hit.MimeType;
+
 			// FIXME: This is evil.  Nautilus should be handling
 			// inode/directory, not just x-directory/normal
-			if (hit.MimeType == "inode/directory")
-				hit.MimeType = "x-directory/normal";
+			if (mimetype == "inode/directory")
+				mimetype = "x-directory/normal";
 
 #if ENABLE_DESKTOP_LAUNCH
 			command = "desktop-launch";
 			expects_uris = true;
-#else		       
+#elif ENABLE_XDG_OPEN
+			command = "xdg-open";
+			expects_uris = true;
+#else
 			GnomeFu.VFSMimeApplication app;
-			app = GnomeFu.GetDefaultAction (hit.MimeType);
+			app = GnomeFu.GetDefaultAction (mimetype);
 			if (app.command != null) {
 				command = app.command;
 				expects_uris = (app.expects_uris != GnomeFu.VFSMimeApplicationArgumentType.Path);
 			}
 #endif			
 			if (command == null) {
-				Console.WriteLine ("Can't open MimeType '{0}'", hit.MimeType);
+				Console.WriteLine ("Can't open MimeType '{0}'", mimetype);
 				return;
 			}
 			
-			if (expects_uris)
-				item = hit.EscapedUri;
-			else
+			if (expects_uris) {
+				// FIXME: I'm not sure that opening the parent
+				// URI (if present) is the right thing to do in
+				// all cases, but it does work for all our
+				// current cases.
+				if (hit.ParentUri != null)
+					item = hit.EscapedParentUri;
+				else
+					item = hit.EscapedUri;
+			} else
 				item = hit.Path;
 
 			// Sometimes the command is 'quoted'
@@ -409,9 +424,14 @@ namespace Search.Tiles {
 
 		public void OpenFromUri (string uri)
                 {
-#if ENABLE_DESKTOP_LAUNCH
+#if ENABLE_DESKTOP_LAUNCH || ENABLE_XDG_OPEN
 			SafeProcess p = new SafeProcess ();
+
+#  if ENABLE_DESKTOP_LAUNCH
 			p.Arguments = new string[] { "desktop-launch", uri };
+#  elif ENABLE_XDG_OPEN
+			p.Arguments = new string[] { "xdg-open", uri };
+#  endif
 
 			try {
 				p.Start ();
