@@ -421,11 +421,9 @@ public class SettingsDialog
 		reader.Close ();
 
 		if (! enabled) {
-			// FIXME: gnome-session has a bug in which autostart overrides
-			// break if Hidden=true is set.
-			writer.WriteLine ("# FIXME: Hidden=true has to be commented out for GNOME autostart to be");
-			writer.WriteLine ("# disabled, but KDE requires it to disable autostart.");
-			writer.WriteLine ("#Hidden=true");
+			writer.WriteLine ("# Setting Hidden=true unintuitively disables autostart on KDE and");
+			writer.WriteLine ("# GNOME >= 2.19.2, but it breaks disabling of autostart in older GNOME.");
+			writer.WriteLine ("Hidden=true");
 			writer.WriteLine ("X-GNOME-Autostart-enabled=false");
 		}
 
@@ -439,11 +437,9 @@ public class SettingsDialog
 		reader.Close ();
 
 		if (! enabled) {
-			// FIXME: gnome-session has a bug in which autostart overrides
-			// break if Hidden=true is set.
-			writer.WriteLine ("# FIXME: Hidden=true has to be commented out for GNOME autostart to be");
-			writer.WriteLine ("# disabled, but KDE requires it to disable autostart.");
-			writer.WriteLine ("#Hidden=true");
+			writer.WriteLine ("# Setting Hidden=true unintuitively disables autostart on KDE and");
+			writer.WriteLine ("# GNOME >= 2.19.2, but it breaks disabling of autostart in older GNOME.");
+			writer.WriteLine ("Hidden=true");
 			writer.WriteLine ("X-GNOME-Autostart-enabled=false");
 		}
 
